@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 22097 $
+ * $Author: rjongbloed $
+ * $Date: 2009-02-21 04:55:10 +0000 (Sat, 21 Feb 2009) $
  */
 
 #ifndef OPAL_OPAL_H
@@ -66,7 +66,7 @@ typedef struct OpalHandleStruct * OpalHandle;
 typedef struct OpalMessage OpalMessage;
 
 
-#define OPAL_C_API_VERSION 14
+#define OPAL_C_API_VERSION 16
 
 
 ///////////////////////////////////////
@@ -355,8 +355,7 @@ typedef enum OpalMessageType {
                                     connection. See the OpalStatusUserInput structure for more information. */
   OpalIndMessageWaiting,        /**<Message Waiting indication. This message is returned in the
                                     OpalGetMessage() function when an MWI is received on any of the supported
-                                    protocols.
-                                */
+                                    protocols. */
   OpalIndMediaStream,           /**<A media stream has started/stopped. This message is returned in the
                                     OpalGetMessage() function when a media stream is started or stopped. See the
                                     OpalStatusMediaStream structure for more information. */
@@ -365,8 +364,19 @@ typedef enum OpalMessageType {
   OpalCmdSetUserData,           /**<Set the user data field associated with a call */
   OpalIndLineAppearance,        /**<Line Appearance indication. This message is returned in the
                                     OpalGetMessage() function when any of the supported protocols indicate that
-                                    the state of a "line" has changed, e.g. free, busy, on hold etc.
-                                */
+                                    the state of a "line" has changed, e.g. free, busy, on hold etc. */
+  OpalCmdStartRecording,        /**<Start recording an active call. See the OpalParamRecording structure
+                                    for more information. */
+  OpalCmdStopRecording,         /**<Stop recording an active call. Only the m_callToken field of the
+                                    OpalMessage union is used. */
+  OpalIndProceeding,            /**<Call has been accepted by remote. This message is returned in the
+                                    OpalGetMessage() function when the underlying protocol states the remote
+                                    endpoint acknowledged that it will route the call. This is distinct from
+                                    OpalIndAlerting in that it is not known at this time if anything is
+                                    ringing. This indication may be used to distinguish between "transport"
+                                    level error, in which case another host may be tried, and that the
+                                    responsibility for finalising the call has moved "upstream". See the
+                                    OpalParamSetUpCall structure for more information. */
   OpalMessageTypeCount
 } OpalMessageType;
 
@@ -441,7 +451,7 @@ typedef enum OpalMediaDataType {
   OpalMediaDataNoChange,      /**< No change to the media data type. */
   OpalMediaDataPayloadOnly,   /**< Indicate only the RTP payload is passed to the
                                    read/write function */
-  OpalMediaDataWithHeader     /**< Indicate the while RTP frame including header is
+  OpalMediaDataWithHeader     /**< Indicate the whole RTP frame including header is
                                    passed to the read/write function */
 } OpalMediaDataType;
 
@@ -612,6 +622,7 @@ typedef struct OpalParamProtocol {
       OpalMessage   command;
       OpalMessage * response;
 
+      // H.323 register with gatekeeper
       memset(&command, 0, sizeof(command));
       command.m_type = OpalCmdRegistration;
       command.m_param.m_registrationInfo.m_protocol = "h323";
@@ -620,7 +631,10 @@ typedef struct OpalParamProtocol {
       command.m_param.m_registrationInfo.m_password = "secret";
       command.m_param.m_registrationInfo.m_timeToLive = 300;
       response = OpalSendMessage(hOPAL, &command);
+      if (response != NULL && response->m_type == OpalCmdRegistration)
+        m_AddressOfRecord = response->m_param.m_registrationInfo.m_identifier
 
+      // SIP register with regstrar/proxy
       memset(&command, 0, sizeof(command));
       command.m_type = OpalCmdRegistration;
       command.m_param.m_registrationInfo.m_protocol = "sip";
@@ -628,15 +642,37 @@ typedef struct OpalParamProtocol {
       command.m_param.m_registrationInfo.m_password = "secret";
       command.m_param.m_registrationInfo.m_timeToLive = 300;
       response = OpalSendMessage(hOPAL, &command);
+      if (response != NULL && response->m_type == OpalCmdRegistration)
+        m_AddressOfRecord = response->m_param.m_registrationInfo.m_identifier
 
+      // unREGISTER
       memset(&command, 0, sizeof(command));
       command.m_type = OpalCmdRegistration;
       command.m_param.m_registrationInfo.m_protocol = "sip";
-      command.m_param.m_registrationInfo.m_identifier = "1501@pbx.local";
-      command.m_param.m_registrationInfo.m_hostName = "1502@pbx.local";
+      command.m_param.m_registrationInfo.m_identifier = m_AddressOfRecord;
+      command.m_param.m_registrationInfo.m_timeToLive = 0;
+      response = OpalSendMessage(hOPAL, &command);
+
+      // Set event package so do SUBSCRIBE
+      memset(&command, 0, sizeof(command));
+      command.m_type = OpalCmdRegistration;
+      command.m_param.m_registrationInfo.m_protocol = "sip";
+      command.m_param.m_registrationInfo.m_identifier = "2012@pbx.local";
+      command.m_param.m_registrationInfo.m_hostName = "sa@pbx.local";
       command.m_param.m_registrationInfo.m_eventPackage = OPAL_LINE_APPEARANCE_EVENT_PACKAGE;
       command.m_param.m_registrationInfo.m_timeToLive = 300;
       response = OpalSendMessage(hOPAL, &command);
+      if (response != NULL && response->m_type == OpalCmdRegistration)
+        m_AddressOfRecord = response->m_param.m_registrationInfo.m_identifier
+
+      // unSUBSCRIBE
+      memset(&command, 0, sizeof(command));
+      command.m_type = OpalCmdRegistration;
+      command.m_param.m_registrationInfo.m_protocol = "sip";
+      command.m_param.m_registrationInfo.m_identifier = m_AddressOfRecord;
+      command.m_param.m_registrationInfo.m_eventPackage = OPAL_LINE_APPEARANCE_EVENT_PACKAGE;
+      command.m_param.m_registrationInfo.m_timeToLive = 0;
+      response = OpalSendMessage(hOPAL, &command);
   */
 typedef struct OpalParamRegistration {
   const char * m_protocol;      /**< Protocol prefix for registration. Currently must be "h323" or
@@ -751,8 +787,8 @@ typedef struct OpalParamSetUpCall {
 
                                    For OpalCmdTransferCall this indicates the party to be
                                    transferred, e.g. "sip:fred@nurk.com". If NULL then
-                                   it is assumed that the party that is not the pc or pots
-                                   connection is to be transferred.
+                                   it is assumed that the party to be transferred is of the
+                                   same "protocol" as the m_partyB field, e.g. "pc" or "sip".
 
                                    For OpalIndAlerting and OpalIndEstablished this indicates
                                    the A-party of the call in progress. */
@@ -773,6 +809,24 @@ typedef struct OpalParamSetUpCall {
                                    returned by the OpalSendMessage() function. The user would
                                    provide the call token for the call being transferred when
                                    OpalCmdTransferCall is being called. */
+  const char * m_alertingType;/**< The type of "distinctive ringing" for the call. The string
+                                   is protocol dependent, so the caller would need to be aware
+                                   of the type of call being made. Some protocols may ignore
+                                   the field completely.
+
+                                   For SIP this corresponds to the string contained in the
+                                   "Alert-Info" header field of the INVITE. This is typically
+                                   a URI for the ring file.
+
+                                   For H.323 this must be a string representation of an
+                                   integer from 0 to 7 which will be contained in the
+                                   Q.931 SIGNAL (0x34) Information Element.
+
+                                   This is only used in OpalCmdSetUpCall to set the string to
+                                   be sent to the remote to change the type of ring the remote
+                                   may emit.
+
+                                   For other indications this field is NULL. */
 } OpalParamSetUpCall;
 
 
@@ -788,6 +842,18 @@ typedef struct OpalStatusIncomingCall {
   const char * m_calledAddress;     ///< URL of called party the remote is trying to contact.
   const char * m_calledPartyNumber; ///< This is the E.164 number of the called party, if available.
   OpalProductDescription m_product; /**< Product description data */
+  const char * m_alertingType;/**< The type of "distinctive ringing" for the call. The string
+                                   is protocol dependent, so the caller would need to be aware
+                                   of the type of call being made. Some protocols may ignore
+                                   the field completely.
+
+                                   For SIP this corresponds to the string contained in the
+                                   "Alert-Info" header field of the INVITE. This is typically
+                                   a URI for the ring file.
+
+                                   For H.323 this must be a string representation of an
+                                   integer from 0 to 7 which will be contained in the
+                                   Q.931 SIGNAL (0x34) Information Element. */
 } OpalStatusIncomingCall;
 
 
@@ -903,6 +969,18 @@ typedef struct OpalStatusLineAppearance {
 } OpalStatusLineAppearance;
 
 
+/**Call recording information for the OpalCmdStartRecording command.
+  */
+typedef struct OpalParamRecording {
+  const char * m_callToken;  ///< Call token for call being cleared.
+  const char * m_file;       /**< File to record into. If NULL then a test is done
+                                  for if recording is currently active. */
+  unsigned     m_channels;   /**< Number of channels in WAV file, 1 for mono (default) or 2 for
+                                  stereo where incoming & outgoing audio are in individual
+                                  channels. */
+} OpalParamRecording;
+
+
 /**Call clearance information for the OpalIndCallCleared indication.
    This is only returned from the OpalGetMessage() function.
   */
@@ -975,8 +1053,8 @@ struct OpalMessage {
     OpalParamProtocol        m_protocol;           ///< Used by OpalCmdSetProtocolParameters
     OpalParamRegistration    m_registrationInfo;   ///< Used by OpalCmdRegistration
     OpalStatusRegistration   m_registrationStatus; ///< Used by OpalIndRegistrationStatus
-    OpalParamSetUpCall       m_callSetUp;          ///< Used by OpalCmdSetUpCall/OpalIndAlerting/OpalIndEstablished
-    const char *             m_callToken;          ///< Used by OpalCmdAnswerCall/OpalCmdHoldcall/OpalCmdRetreiveCall
+    OpalParamSetUpCall       m_callSetUp;          ///< Used by OpalCmdSetUpCall/OpalIndProceeding/OpalIndAlerting/OpalIndEstablished
+    const char *             m_callToken;          ///< Used by OpalCmdAnswerCall/OpalCmdHoldcall/OpalCmdRetreiveCall/OpalCmdStopRecording
     OpalStatusIncomingCall   m_incomingCall;       ///< Used by OpalIndIncomingCall
     OpalStatusUserInput      m_userInput;          ///< Used by OpalIndUserInput
     OpalStatusMessageWaiting m_messageWaiting;     ///< Used by OpalIndMessageWaiting
@@ -985,6 +1063,7 @@ struct OpalMessage {
     OpalParamCallCleared     m_clearCall;          ///< Used by OpalCmdClearCall
     OpalStatusMediaStream    m_mediaStream;        ///< Used by OpalIndMediaStream/OpalCmdMediaStream
     OpalParamSetUserData     m_setUserData;        ///< Used by OpalCmdSetUserData
+    OpalParamRecording       m_recording;          ///< Used by OpalCmdStartRecording
   } m_param;
 };
 
