@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 22053 $
+ * $Author: csoutheren $
+ * $Date: 2009-02-12 07:30:50 +0000 (Thu, 12 Feb 2009) $
  */
 
 #ifndef OPAL_OPAL_MEDIAFMT_H
@@ -128,10 +128,19 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
       const char * protocol = NULL                ///<  protocol to be valid for (if NULL, then all)
     ) const;
 
-    /**Get a format position in the list matching the wildcard.
-       The wildcard string is a simple substring match using the '*'
-       character. For example: "G.711*" would match "G.711-uLaw-64k" and
-       "G.711-ALaw-64k".
+    /**Get a position in the list of the first entry matching the wildcard.
+       The wildcard string is a string match using several special
+       characters.
+       
+       The '*' character indicates substrings, for example: "G.711*" would
+       match "G.711-uLaw-64k" and "G.711-ALaw-64k".
+
+       The '@' character indicates a type of media format, Currently only
+       "@audio" and "@video" is supported.
+
+       The '!' character indicates a negative test. That is the first entry
+       that does NOT match the string is returned. The string after the '!'
+       may contain '*' and '@' characters.
 
        Returns P_MAX_INDEX if not in list.
       */
@@ -156,18 +165,30 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
     ) const { return FindFormat(wildcard) != end(); }
 
     /**Remove all the formats specified.
+       Each string in the array is checked using the wildcard matching algorithm in
+       FindFormat().
       */
     void Remove(
       const PStringArray & mask
     );
 
     /**Reorder the formats in the list.
-       The order variable is an array of wildcards and the list is reordered
+       The order variable is an array of names and the list is reordered
        according to the order in that array.
+
+       Each string in the array is checked using the wildcard matching algorithm in
+       FindFormat().
       */
     void Reorder(
       const PStringArray & order
     );
+
+    /**Determine if a list contains a specific media type
+      */
+    bool HasType(
+      const OpalMediaType & type,
+      bool mustBeTransportable = true
+    ) const;
   //@}
 
   private:
@@ -510,8 +531,8 @@ class OpalMediaFormatInternal : public PObject
     virtual PObject * Clone() const;
     virtual void PrintOn(ostream & strm) const;
 
-    virtual PBoolean IsValid() const { return rtpPayloadType < RTP_DataFrame::IllegalPayloadType && !formatName.IsEmpty(); }
-    virtual PBoolean IsTransportable() const { return rtpPayloadType < RTP_DataFrame::MaxPayloadType && !rtpEncodingName.IsEmpty(); }
+    virtual bool IsValid() const;
+    virtual bool IsTransportable() const;
 
     virtual PStringToString GetOptions() const;
     virtual bool GetOptionValue(const PString & name, PString & value) const;
@@ -545,6 +566,7 @@ class OpalMediaFormatInternal : public PObject
     PMutex                       media_format_mutex;
     PSortedList<OpalMediaOption> options;
     time_t                       codecVersionTime;
+    bool                         forceIsTransportable;
 
   friend bool operator==(const char * other, const OpalMediaFormat & fmt);
   friend bool operator!=(const char * other, const OpalMediaFormat & fmt);
@@ -569,6 +591,9 @@ class OpalMediaFormat : public PContainer
   PCONTAINERINFO(OpalMediaFormat, PContainer);
 
   public:
+    static const PString & BandwidthTIASOption();
+    static const PString & MaxPacketRateOption();
+
     /**Default constructor creates a PCM-16 media format.
       */
     OpalMediaFormat(
@@ -1011,6 +1036,7 @@ class OpalMediaFormat : public PContainer
 
 #if OPAL_H323
     static const PString & MediaPacketizationOption();
+    static const PString & MediaPacketizationsOption();
 #endif
 
   private:
@@ -1113,6 +1139,7 @@ class OpalVideoFormat : public OpalMediaFormat
     static const PString & RateControlEnableOption();
     static const PString & RateControlWindowSizeOption();
     static const PString & RateControlMaxFramesSkipOption();
+    static const PString & RateControllerBitRateScalerOption();
 };
 #endif
 
@@ -1120,10 +1147,17 @@ class OpalVideoFormat : public OpalMediaFormat
 
 #define OPAL_PCM16          "PCM-16"
 #define OPAL_PCM16_16KHZ    "PCM-16-16kHz"
+#define OPAL_PCM16_32KHZ    "PCM-16-32kHz"
+#define OPAL_PCM16_48KHZ    "PCM-16-48kHz"
 #define OPAL_L16_MONO_8KHZ  "Linear-16-Mono-8kHz"
 #define OPAL_L16_MONO_16KHZ "Linear-16-Mono-16kHz"
+#define OPAL_L16_MONO_32KHZ "Linear-16-Mono-32kHz"
+#define OPAL_L16_MONO_48KHZ "Linear-16-Mono-48kHz"
 #define OPAL_G711_ULAW_64K  "G.711-uLaw-64k"
 #define OPAL_G711_ALAW_64K  "G.711-ALaw-64k"
+#define OPAL_G722           "G.722"
+#define OPAL_G7221          "G.722.1"
+#define OPAL_G7222          "G.722.2"
 #define OPAL_G726_40K       "G.726-40K"
 #define OPAL_G726_32K       "G.726-32K"
 #define OPAL_G726_24K       "G.726-24K"
@@ -1148,15 +1182,21 @@ class OpalVideoFormat : public OpalMediaFormat
 #define OPAL_RFC2833        "UserInput/RFC2833"
 #define OPAL_CISCONSE       "NamedSignalEvent"
 #define OPAL_T38            "T.38"
-#define OPAL_H224            "H.224"
 
 extern const OpalAudioFormat & GetOpalPCM16();
 extern const OpalAudioFormat & GetOpalPCM16_16KHZ();
+extern const OpalAudioFormat & GetOpalPCM16_32KHZ();
+extern const OpalAudioFormat & GetOpalPCM16_48KHZ();
 extern const OpalAudioFormat & GetOpalL16_MONO_8KHZ();
 extern const OpalAudioFormat & GetOpalL16_MONO_16KHZ();
+extern const OpalAudioFormat & GetOpalL16_MONO_32KHZ();
+extern const OpalAudioFormat & GetOpalL16_MONO_48KHZ();
 extern const OpalAudioFormat & GetOpalL16_STEREO_48KHZ();
 extern const OpalAudioFormat & GetOpalG711_ULAW_64K();
 extern const OpalAudioFormat & GetOpalG711_ALAW_64K();
+extern const OpalAudioFormat & GetOpalG722();
+extern const OpalAudioFormat & GetOpalG7221();
+extern const OpalMediaFormat & GetOpalG7222();
 extern const OpalAudioFormat & GetOpalG726_40K();
 extern const OpalAudioFormat & GetOpalG726_32K();
 extern const OpalAudioFormat & GetOpalG726_24K();
@@ -1171,22 +1211,31 @@ extern const OpalAudioFormat & GetOpalG7231_5k3();
 extern const OpalAudioFormat & GetOpalG7231A_6k3();
 extern const OpalAudioFormat & GetOpalG7231A_5k3();
 extern const OpalAudioFormat & GetOpalGSM0610();
-extern const OpalAudioFormat & GetOpalGSMAMR();
+extern const OpalMediaFormat & GetOpalGSMAMR();
 extern const OpalMediaFormat & GetOpaliLBC();
 
 extern const OpalMediaFormat & GetOpalRFC2833();
+
 #if OPAL_T38_CAPABILITY
 extern const OpalMediaFormat & GetOpalCiscoNSE();
 extern const OpalMediaFormat & GetOpalT38();
 #endif
 
+
 #define OpalPCM16          GetOpalPCM16()
 #define OpalPCM16_16KHZ    GetOpalPCM16_16KHZ()
+#define OpalPCM16_32KHZ    GetOpalPCM16_32KHZ()
+#define OpalPCM16_48KHZ    GetOpalPCM16_48KHZ()
 #define OpalL16_MONO_8KHZ  GetOpalL16_MONO_8KHZ()
 #define OpalL16_MONO_16KHZ GetOpalL16_MONO_16KHZ()
+#define OpalL16_MONO_32KHZ GetOpalL16_MONO_32KHZ()
+#define OpalL16_MONO_48KHZ GetOpalL16_MONO_48KHZ()
 #define OpalL16_STEREO_48KHZ GetOpalL16_STEREO_48KHZ()
 #define OpalG711_ULAW_64K  GetOpalG711_ULAW_64K()
 #define OpalG711_ALAW_64K  GetOpalG711_ALAW_64K()
+#define OpalG722           GetOpalG722()
+#define OpalG7221          GetOpalG7221()
+#define OpalG7222          GetOpalG7222()
 #define OpalG726_40K       GetOpalG726_40K()
 #define OpalG726_32K       GetOpalG726_32K()
 #define OpalG726_24K       GetOpalG726_24K()
@@ -1212,6 +1261,22 @@ extern const OpalMediaFormat & GetOpalT38();
 #define OpalG711uLaw       OpalG711_ULAW_64K
 #define OpalG711ALaw       OpalG711_ALAW_64K
 
+#define OPAL_T140             "T.140"
+#define OpalT140              GetOpalT140()
+extern const OpalMediaFormat & GetOpalT140();
+
+#if OPAL_HAS_MSRP
+#define OPAL_MSRP           "MSRP"
+#define OpalMSRP            GetOpalMSRP()
+extern const OpalMediaFormat & GetOpalMSRP();
+#endif
+
+#if OPAL_HAS_SIPIM
+#define OPAL_SIPIM             "SIP-IM"
+#define OpalSIPIM              GetOpalSIPIM()
+extern const OpalMediaFormat & GetOpalSIPIM();
+#endif
+
 #ifdef _MSC_VER
 #if _MSC_VER < 1300
 #pragma warning(default:4663)
