@@ -164,7 +164,7 @@ FFMPEGLibrary::FFMPEGLibrary(CodecID codec)
   isLoadedOK = false;
 }
 
-bool FFMPEGLibrary::Load()
+bool FFMPEGLibrary::Load(int ver)
 {
   WaitAndSignal m(processLock);      
   if (IsLoaded())
@@ -225,10 +225,10 @@ bool FFMPEGLibrary::Load()
     }
   }
 
-    if (!GetFunction("register_avcodec", (Function &)Favcodec_register)) {
-      TRACE (1, _codecString << "\tDYNA\tFailed to load register_avcodec");
-      return false;
-    }
+  if (!GetFunction("register_avcodec", (Function &)Favcodec_register)) {
+    TRACE (1, _codecString << "\tDYNA\tFailed to load register_avcodec");
+    return false;
+  }
   
   if (!GetFunction("avcodec_find_encoder", (Function &)Favcodec_find_encoder)) {
     TRACE (1, _codecString << "\tDYNA\tFailed to load avcodec_find_encoder");
@@ -270,6 +270,12 @@ bool FFMPEGLibrary::Load()
     return false;
   }
 
+  Favcodec_set_dimensions = NULL;
+  if (ver > 0 && !GetFunction("avcodec_set_dimensions", (Function &)Favcodec_set_dimensions)) {
+    TRACE (1, _codecString << "\tDYNA\tFailed to load avcodec_set_dimensions");
+    return false;
+  }
+
   if (!GetFunction("av_free", (Function &)Favcodec_free)) {
     TRACE (1, _codecString << "\tDYNA\tFailed to load av_free");
     return false;
@@ -381,6 +387,8 @@ int FFMPEGLibrary::AvcodecOpen(AVCodecContext *ctx, AVCodec *codec)
 
 int FFMPEGLibrary::AvcodecClose(AVCodecContext *ctx)
 {
+  WaitAndSignal m(processLock);
+
   WITH_ALIGNED_STACK({
     return Favcodec_close(ctx);
   });
@@ -417,6 +425,14 @@ void FFMPEGLibrary::AvcodecFree(void * ptr)
   });
 }
 
+void FFMPEGLibrary::AvSetDimensions(AVCodecContext *s, int width, int height)
+{
+  WITH_ALIGNED_STACK({
+    Favcodec_set_dimensions(s, width, height);
+  });
+}
+  
+
 void FFMPEGLibrary::AvLogSetLevel(int level)
 {
   WITH_ALIGNED_STACK({
@@ -446,3 +462,4 @@ bool FFMPEGLibrary::IsLoaded()
 {
   return isLoadedOK;
 }
+
