@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21231 $
+ * $Revision: 21969 $
  * $Author: rjongbloed $
- * $Date: 2008-10-03 11:48:42 +0000 (Fri, 03 Oct 2008) $
+ * $Date: 2009-01-30 23:05:54 +0000 (Fri, 30 Jan 2009) $
  */
 
 #include <ptlib.h>
@@ -1475,180 +1475,6 @@ PString H323_G711Capability::GetFormatName() const
 }
 
 
-/////////////////////////////////////////////////////////////////////////////
-
-#if 0
-
-H323_G728Capability::H323_G728Capability()
-  : H323AudioCapability(240, 30) // 240ms max, 30ms desired
-{
-}
-
-
-PObject * H323_G728Capability::Clone() const
-{
-  return new H323_G728Capability(*this);
-}
-
-
-unsigned H323_G728Capability::GetSubType() const
-{
-  return H245_AudioCapability::e_g728;
-}
-
-
-PString H323_G728Capability::GetFormatName() const
-{
-  return OpalG728;
-}
-
-
-/////////////////////////////////////////////////////////////////////////////
-
-static const char * const G729Name[4] = {
-  OPAL_G729,
-  OPAL_G729A,
-  OPAL_G729B,
-  OPAL_G729AB
-};
-
-H323_G729Capability::H323_G729Capability(Mode m)
-{
-  mode = m;
-}
-
-
-PObject * H323_G729Capability::Clone() const
-{
-  return new H323_G729Capability(*this);
-}
-
-
-unsigned H323_G729Capability::GetSubType() const
-{
-  static const unsigned G729SubType[4] = {
-    H245_AudioCapability::e_g729,
-    H245_AudioCapability::e_g729AnnexA,
-    H245_AudioCapability::e_g729wAnnexB,
-    H245_AudioCapability::e_g729AnnexAwAnnexB
-  };
-  return G729SubType[mode];
-}
-
-
-PString H323_G729Capability::GetFormatName() const
-{
-  return G729Name[mode];
-}
-
-
-/////////////////////////////////////////////////////////////////////////////
-
-H323_G7231Capability::H323_G7231Capability(PBoolean sid)
-{
-  allowSIDFrames = sid;
-}
-
-
-PObject * H323_G7231Capability::Clone() const
-{
-  return new H323_G7231Capability(*this);
-}
-
-
-unsigned H323_G7231Capability::GetSubType() const
-{
-  return H245_AudioCapability::e_g7231;
-}
-
-
-PString H323_G7231Capability::GetFormatName() const
-{
-  return OpalG7231_6k3;
-}
-
-
-PBoolean H323_G7231Capability::OnSendingPDU(H245_AudioCapability & pdu,
-                                        unsigned packetSize) const
-{
-  pdu.SetTag(H245_AudioCapability::e_g7231);
-  H245_AudioCapability_g7231 & g7231 = pdu;
-  g7231.m_maxAl_sduAudioFrames = packetSize;
-  g7231.m_silenceSuppression = allowSIDFrames;
-  return PTrue;
-}
-
-
-PBoolean H323_G7231Capability::OnReceivedPDU(const H245_AudioCapability & pdu,
-                                         unsigned & packetSize)
-{
-  if (pdu.GetTag() != H245_AudioCapability::e_g7231)
-    return PFalse;
-
-  const H245_AudioCapability_g7231 & g7231 = pdu;
-  packetSize = g7231.m_maxAl_sduAudioFrames;
-  allowSIDFrames = g7231.m_silenceSuppression;
-  return PTrue;
-}
-
-
-///////////////////////////////////////////////////////////////////////////////
-
-H323_GSM0610Capability::H323_GSM0610Capability()
-{
-}
-
-
-PObject * H323_GSM0610Capability::Clone() const
-{
-  return new H323_GSM0610Capability(*this);
-}
-
-
-unsigned H323_GSM0610Capability::GetSubType() const
-{
-  return H245_AudioCapability::e_gsmFullRate;
-}
-
-
-PString H323_GSM0610Capability::GetFormatName() const
-{
-  return OpalGSM0610;
-}
-
-
-void H323_GSM0610Capability::SetTxFramesInPacket(unsigned frames)
-{
-  H323AudioCapability::SetTxFramesInPacket(frames > 7 ? 7 : frames);
-}
-
-
-PBoolean H323_GSM0610Capability::OnSendingPDU(H245_AudioCapability & cap,
-                                          unsigned packetSize) const
-{
-  cap.SetTag(H245_AudioCapability::e_gsmFullRate);
-
-  H245_GSMAudioCapability & gsm = cap;
-  gsm.m_audioUnitSize = packetSize*33;
-  return PTrue;
-}
-
-
-PBoolean H323_GSM0610Capability::OnReceivedPDU(const H245_AudioCapability & cap,
-                                           unsigned & packetSize)
-{
-  if (cap.GetTag() != H245_AudioCapability::e_gsmFullRate)
-    return PFalse;
-
-  const H245_GSMAudioCapability & gsm = cap;
-  packetSize = gsm.m_audioUnitSize/33;
-  if (packetSize == 0)
-    packetSize = 1;
-  return PTrue;
-}
-
-#endif   // 0
-
 /////////////////////////////////////////////////////////////////////////////
 
 static const char * const UIISubTypeNames[H323_UserInputCapability::NumSubTypes] = {
@@ -1883,6 +1709,7 @@ H323Capabilities::H323Capabilities(const H323Connection & connection,
    *  version of the codec against possibly multiple codec with the same 'subtype' such as
    *  e_h263VideoCapability.
    */
+  mediaPacketizations += "RFC2190";  // Always supported
   const H245_MultiplexCapability * muxCap = NULL;
   if (pdu.HasOptionalField(H245_TerminalCapabilitySet::e_multiplexCapability)) {
     muxCap = &pdu.m_multiplexCapability;
@@ -1897,8 +1724,8 @@ H323Capabilities::H323Capabilities(const H323Connection & connection,
         for (PINDEX i = 0; i < mediaPacket.m_rtpPayloadType.GetSize(); i++) {
           PString mediaPacketization = H323GetRTPPacketization(mediaPacket.m_rtpPayloadType[i]);
           if (!mediaPacketization.IsEmpty()) {
-              mediaPacketizations += mediaPacketization;
-              PTRACE(4, "H323\tH323Capabilities(ctor) Appended mediaPacketization=" << mediaPacketization << ", mediaPacketization count=" << mediaPacketizations.GetSize());
+            mediaPacketizations += mediaPacketization;
+            PTRACE(4, "H323\tH323Capabilities(ctor) Appended mediaPacketization=" << mediaPacketization << ", mediaPacketization count=" << mediaPacketizations.GetSize());
           }
         } // for ... m_rtpPayloadType.GetSize()
       } // e_rtpPayloadType
@@ -1913,6 +1740,7 @@ H323Capabilities::H323Capabilities(const H323Connection & connection,
       allCapabilities.Add(allCapabilities.Copy(localCapabilities[c]));
     allCapabilities.AddAllCapabilities(connection.GetEndPoint(), 0, 0, "*");
     H323_UserInputCapability::AddAllCapabilities(allCapabilities, P_MAX_INDEX, P_MAX_INDEX);
+    allCapabilities.SetMediaPacketizations(mediaPacketizations);
 
     for (PINDEX i = 0; i < pdu.m_capabilityTable.GetSize(); i++) {
       if (pdu.m_capabilityTable[i].HasOptionalField(H245_CapabilityTableEntry::e_capability)) {
@@ -1920,6 +1748,33 @@ H323Capabilities::H323Capabilities(const H323Connection & connection,
         if (capability != NULL) {
           H323Capability * copy = (H323Capability *)capability->Clone();
           copy->SetCapabilityNumber(pdu.m_capabilityTable[i].m_capabilityTableEntryNumber);
+          if (mediaPacketizations.GetSize() != 0) { // also update the mediaPacketizations option
+            OpalMediaFormat & mediaFormat = copy->GetWritableMediaFormat();
+            PString packetizationString = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
+                                          mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
+            if (!packetizationString.IsEmpty()) {
+              // remove packetizations not signaled by the remote party
+              PStringArray packetizations = packetizationString.Tokenise(",");
+              for (PINDEX j = packetizations.GetSize(); j > 0; j--) {
+                if (!mediaPacketizations.Contains(packetizations[j-1])) {
+                  packetizations.RemoveAt(j-1);
+                }
+              }
+
+              // construct the new packetization string
+              packetizationString.MakeEmpty();
+              if (packetizations.GetSize() == 0) { // should not happen actually
+                mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationsOption(), PString::Empty());
+                mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(),  PString::Empty());
+              } else {
+                packetizationString = packetizations[0];
+                for (PINDEX j = 1; j < packetizations.GetSize(); j++)
+                  packetizationString += "," + packetizations[j];
+                mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationsOption(), packetizationString);
+                mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(),  packetizations[0]);
+              }
+            }
+          }
           if (copy->OnReceivedPDU(pdu.m_capabilityTable[i].m_capability) &&
               copy->GetWritableMediaFormat().ToNormalisedOptions())
             table.Append(copy);
@@ -2057,9 +1912,14 @@ PINDEX H323Capabilities::AddMediaFormat(PINDEX descriptorNum,
     if (capability != NULL) {
       capability->GetWritableMediaFormat() = mediaFormat;
       reply = SetCapability(descriptorNum, simultaneous, capability);
-      PString packetization = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption());
-      if (!packetization.IsEmpty())
-        mediaPacketizations += packetization;
+      PString packetizationString = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
+                                    mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
+      if (!packetizationString.IsEmpty()) {
+        PStringArray packetizations = packetizationString.Tokenise(",");
+        for (PINDEX i = 0; i < packetizations.GetSize(); i++) {
+          mediaPacketizations += packetizations[i];
+        }
+      }
     }
   }
 
@@ -2271,32 +2131,14 @@ H323Capability * H323Capabilities::FindCapability(const H245_Capability & cap) c
   for (PINDEX i = 0; i < table.GetSize(); i++) {
     H323Capability & capability = table[i];
 
-    /** If mediaPacketization information is available, use this with the FindCapability() logic.
-     *  Certain codecs, such as H.263, need additional information in order to match the specific
-     *  version of the codec against possibly multiple codec with the same 'subtype' such as
-     *  e_h263VideoCapability.
-     */
-    if (!mediaPacketizations.IsEmpty()) {
-      PString capabilitiesPacketization = capability.GetMediaFormat().GetOptionString(OpalMediaFormat::MediaPacketizationOption());
-      if (!capabilitiesPacketization.IsEmpty()) {
-        PINDEX j;
-        for (j = 0; j < mediaPacketizations.GetSize(); j++) {
-          if (mediaPacketizations.GetKeyAt(j) == capabilitiesPacketization)
-            break;
-        }
-        if (j >= mediaPacketizations.GetSize())
-          return false;
-      }
-    }
-
+    bool found = false;
     switch (cap.GetTag()) {
       case H245_Capability::e_receiveAudioCapability :
       case H245_Capability::e_transmitAudioCapability :
       case H245_Capability::e_receiveAndTransmitAudioCapability :
         if (capability.GetMainType() == H323Capability::e_Audio) {
           const H245_AudioCapability & audio = cap;
-          if (capability.IsMatch(audio))
-            return &capability;
+          found = capability.IsMatch(audio);
         }
         break;
 
@@ -2305,8 +2147,7 @@ H323Capability * H323Capabilities::FindCapability(const H245_Capability & cap) c
       case H245_Capability::e_receiveAndTransmitVideoCapability :
         if (capability.GetMainType() == H323Capability::e_Video) {
           const H245_VideoCapability & video = cap;
-          if (capability.IsMatch(video))
-            return &capability;
+          found = capability.IsMatch(video);
         }
         break;
 
@@ -2315,8 +2156,7 @@ H323Capability * H323Capabilities::FindCapability(const H245_Capability & cap) c
       case H245_Capability::e_receiveAndTransmitDataApplicationCapability :
         if (capability.GetMainType() == H323Capability::e_Data) {
           const H245_DataApplicationCapability & data = cap;
-          if (capability.IsMatch(data.m_application))
-            return &capability;
+          found = capability.IsMatch(data.m_application);
         }
         break;
 
@@ -2325,14 +2165,40 @@ H323Capability * H323Capabilities::FindCapability(const H245_Capability & cap) c
       case H245_Capability::e_receiveAndTransmitUserInputCapability :
         if (capability.GetMainType() == H323Capability::e_UserInput) {
           const H245_UserInputCapability & ui = cap;
-          if (capability.IsMatch(ui))
-            return &capability;
+          found = capability.IsMatch(ui);
         }
         break;
 
       case H245_Capability::e_receiveRTPAudioTelephonyEventCapability :
         return FindCapability(H323Capability::e_UserInput, SignalToneRFC2833_SubType);
     }
+
+    if (found) {
+      if (mediaPacketizations.IsEmpty())
+        return &capability;
+
+      /** If mediaPacketization information is available, use this with the FindCapability() logic.
+       *  Certain codecs, such as H.263, need additional information in order to match the specific
+       *  version of the codec against possibly multiple codec with the same 'subtype' such as
+       *  e_h263VideoCapability.
+       */
+      OpalMediaFormat mediaFormat = capability.GetMediaFormat();
+      PString packetizationString = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
+                                    mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
+      if (packetizationString.IsEmpty())
+        return &capability;
+
+      // require at least one of the media format's packetizations to be in the mediaPacketizations list
+      PStringArray packetizations = packetizationString.Tokenise(",");
+      for (PINDEX j = 0; j < packetizations.GetSize(); j++) {
+        for (PINDEX k = 0; k < mediaPacketizations.GetSize(); k++) {
+          if (mediaPacketizations.GetKeyAt(k) == packetizations[j])
+            return &capability;
+        }
+      }
+
+      PTRACE(4, "H323\tUnsupported media packetization " << packetizationString << ", not using capability " << cap);
+    }
   }
 
 #if PTRACING
@@ -2547,18 +2413,8 @@ void H323Capabilities::BuildPDU(const H323Connection & connection,
       capability.GetWritableMediaFormat().ToCustomisedOptions();
       capability.OnSendingPDU(entry.m_capability);
 
-      h225_0.m_mediaPacketizationCapability.m_rtpPayloadType.SetSize(rtpPacketizationCount+1);
-      if (H323SetRTPPacketization(h225_0.m_mediaPacketizationCapability.m_rtpPayloadType[rtpPacketizationCount],
-                                                    capability.GetMediaFormat(), RTP_DataFrame::IllegalPayloadType)) {
-        // Check if already in list
-        PINDEX test;
-        for (test = 0; test < rtpPacketizationCount; test++) {
-          if (h225_0.m_mediaPacketizationCapability.m_rtpPayloadType[test] == h225_0.m_mediaPacketizationCapability.m_rtpPayloadType[rtpPacketizationCount])
-            break;
-        }
-        if (test == rtpPacketizationCount)
-          rtpPacketizationCount++;
-      }
+      H323SetRTPPacketization(h225_0.m_mediaPacketizationCapability.m_rtpPayloadType, rtpPacketizationCount,
+                              capability.GetMediaFormat(), RTP_DataFrame::IllegalPayloadType);
     }
   }
 
