@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21626 $
+ * $Revision: 22096 $
  * $Author: rjongbloed $
- * $Date: 2008-11-26 02:09:35 +0000 (Wed, 26 Nov 2008) $
+ * $Date: 2009-02-21 03:25:23 +0000 (Sat, 21 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -524,7 +524,7 @@ PBoolean OpalLineConnection::SetAlerting(const PString & /*calleeName*/, PBoolea
 {
   PTRACE(3, "LID Con\tSetAlerting " << *this);
 
-  if (GetPhase() != SetUpPhase) 
+  if (GetPhase() >= AlertingPhase) 
     return false;
 
   // switch phase 
