@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 22086 $
+ * $Author: csoutheren $
+ * $Date: 2009-02-18 03:37:22 +0000 (Wed, 18 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -73,8 +73,12 @@ static class InstantiateMe
 
 AUDIO_FORMAT(PCM16,          MaxPayloadType, "",     16, 8,  240,  0, 256,  8000);
 AUDIO_FORMAT(PCM16_16KHZ,    MaxPayloadType, "",     32,16,  240,  0, 256, 16000);
+AUDIO_FORMAT(PCM16_32KHZ,    MaxPayloadType, "",     64,32,  240,  0, 256, 32000);
+AUDIO_FORMAT(PCM16_48KHZ,    MaxPayloadType, "",     96,48,  240,  0, 256, 48000);
 AUDIO_FORMAT(L16_MONO_8KHZ,  L16_Mono,       "L16",  16, 8,  240, 30, 256,  8000);
 AUDIO_FORMAT(L16_MONO_16KHZ, L16_Mono,       "L16",  32,16,  240, 30, 256, 16000);
+AUDIO_FORMAT(L16_MONO_32KHZ, L16_Mono,       "L16",  64,32,  240, 30, 256, 32000);
+AUDIO_FORMAT(L16_MONO_48KHZ, L16_Mono,       "L16",  96,48,  240, 30, 256, 48000);
 AUDIO_FORMAT(G711_ULAW_64K,  PCMU,           "PCMU",  8, 8,  240, 20, 256,  8000);
 AUDIO_FORMAT(G711_ALAW_64K,  PCMA,           "PCMA",  8, 8,  240, 20, 256,  8000);
 
@@ -115,33 +119,41 @@ const OpalAudioFormat & GetOpalL16_STEREO_48KHZ()
 
 const OpalMediaFormat & GetOpalRFC2833()
 {
-  static const OpalMediaFormat RFC2833(
-    OPAL_RFC2833,
-    "userinput",
-    (RTP_DataFrame::PayloadTypes)101,  // Set to this for Cisco compatibility
-    "telephone-event",
-    true,   // Needs jitter
-    32*(1000/50), // bits/sec  (32 bits every 50ms)
-    4,      // bytes/frame
-    150*8,  // 150 millisecond
-    OpalMediaFormat::AudioClockRate
-  );
+  static struct OpalRFC2833MediaFormat : public OpalMediaFormat {
+    OpalRFC2833MediaFormat()
+      : OpalMediaFormat(OPAL_RFC2833,
+                        "userinput",
+                        (RTP_DataFrame::PayloadTypes)101,  // Set to this for Cisco compatibility
+                        "telephone-event",
+                        true,   // Needs jitter
+                        32*(1000/50), // bits/sec  (32 bits every 50ms)
+                        4,      // bytes/frame
+                        150*8,  // 150 millisecond
+                        OpalMediaFormat::AudioClockRate)
+    {
+      AddOption(new OpalMediaOptionString("FMTP", false, "0-16,32,36"));
+    }
+  } const RFC2833;
   return RFC2833;
 }
 
 const OpalMediaFormat & GetOpalCiscoNSE()
 {
-  static const OpalMediaFormat CiscoNSE(
-    OPAL_CISCONSE,
-    "userinput",
-    (RTP_DataFrame::PayloadTypes)100,  // Set to this for Cisco compatibility
-    "NSE",
-    true,   // Needs jitter
-    32*(1000/50), // bits/sec  (32 bits every 50ms)
-    4,      // bytes/frame
-    150*8,  // 150 millisecond
-    OpalMediaFormat::AudioClockRate
-  );
+  static struct OpalCiscoNSEMediaFormat : public OpalMediaFormat {
+    OpalCiscoNSEMediaFormat()
+      : OpalMediaFormat(OPAL_CISCONSE,
+                        "userinput",
+                        (RTP_DataFrame::PayloadTypes)100,  // Set to this for Cisco compatibility
+                        "NSE",
+                        true,   // Needs jitter
+                        32*(1000/50), // bits/sec  (32 bits every 50ms)
+                        4,      // bytes/frame
+                        150*8,  // 150 millisecond
+                        OpalMediaFormat::AudioClockRate)
+    {
+      AddOption(new OpalMediaOptionString("FMTP", false, "192,193"));
+    }
+  } const CiscoNSE;
   return CiscoNSE;
 }
 
@@ -613,14 +625,18 @@ void OpalMediaOptionOctets::SetValue(const BYTE * data, PINDEX length)
 
 /////////////////////////////////////////////////////////////////////////////
 
-const PString & OpalMediaFormat::NeedsJitterOption()  { static PString s = PLUGINCODEC_OPTION_NEEDS_JITTER;   return s; }
-const PString & OpalMediaFormat::MaxFrameSizeOption() { static PString s = PLUGINCODEC_OPTION_MAX_FRAME_SIZE; return s; }
-const PString & OpalMediaFormat::FrameTimeOption()    { static PString s = PLUGINCODEC_OPTION_FRAME_TIME;     return s; }
-const PString & OpalMediaFormat::ClockRateOption()    { static PString s = PLUGINCODEC_OPTION_CLOCK_RATE;     return s; }
-const PString & OpalMediaFormat::MaxBitRateOption()   { static PString s = PLUGINCODEC_OPTION_MAX_BIT_RATE;   return s; }
-const PString & OpalMediaFormat::TargetBitRateOption(){ static PString s = PLUGINCODEC_OPTION_TARGET_BIT_RATE; return s; }
+const PString & OpalMediaFormat::NeedsJitterOption()   { static PString s = PLUGINCODEC_OPTION_NEEDS_JITTER;    return s; }
+const PString & OpalMediaFormat::MaxFrameSizeOption()  { static PString s = PLUGINCODEC_OPTION_MAX_FRAME_SIZE;  return s; }
+const PString & OpalMediaFormat::FrameTimeOption()     { static PString s = PLUGINCODEC_OPTION_FRAME_TIME;      return s; }
+const PString & OpalMediaFormat::ClockRateOption()     { static PString s = PLUGINCODEC_OPTION_CLOCK_RATE;      return s; }
+const PString & OpalMediaFormat::MaxBitRateOption()    { static PString s = PLUGINCODEC_OPTION_MAX_BIT_RATE;    return s; }
+const PString & OpalMediaFormat::TargetBitRateOption() { static PString s = PLUGINCODEC_OPTION_TARGET_BIT_RATE; return s; }
+const PString & OpalMediaFormat::BandwidthTIASOption() { static PString s = PLUGINCODEC_OPTION_TIAS;            return s; }
+const PString & OpalMediaFormat::MaxPacketRateOption() { static PString s = PLUGINCODEC_OPTION_MAXPRATE;        return s; }
+
 #if OPAL_H323
-const PString & OpalMediaFormat::MediaPacketizationOption(){ static PString s = PLUGINCODEC_MEDIA_PACKETIZATION; return s; }
+const PString & OpalMediaFormat::MediaPacketizationOption()  { static PString s = PLUGINCODEC_MEDIA_PACKETIZATION;  return s; }
+const PString & OpalMediaFormat::MediaPacketizationsOption() { static PString s = PLUGINCODEC_MEDIA_PACKETIZATIONS; return s; }
 #endif
 
 
@@ -866,7 +882,7 @@ OpalMediaFormatInternal::OpalMediaFormatInternal(const char * fullName,
                                                  unsigned ft,
                                                  unsigned cr,
                                                  time_t ts)
-  : formatName(fullName), mediaType(_mediaType)
+  : formatName(fullName), mediaType(_mediaType), forceIsTransportable(false)
 {
   codecVersionTime = ts;
   rtpPayloadType = pt;
@@ -884,11 +900,20 @@ OpalMediaFormatInternal::OpalMediaFormatInternal(const char * fullName,
     AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::FrameTimeOption(), true, OpalMediaOption::NoMerge, ft));
 
   if (cr > 0)
-    AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::ClockRateOption(), true, OpalMediaOption::AlwaysMerge, cr));
+    AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::ClockRateOption(), true, OpalMediaOption::NoMerge, cr));
+
+  AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::BandwidthTIASOption(), false, OpalMediaOption::MinMerge, bw, 1, bw));
 
   // assume non-dynamic payload types are correct and do not need deconflicting
-  if (rtpPayloadType < RTP_DataFrame::DynamicBase || rtpPayloadType >= RTP_DataFrame::MaxPayloadType)
+  if (rtpPayloadType < RTP_DataFrame::DynamicBase || rtpPayloadType >= RTP_DataFrame::MaxPayloadType) {
+    if (rtpPayloadType == RTP_DataFrame::MaxPayloadType && 
+        rtpEncodingName.GetLength() > 0 &&
+        rtpEncodingName[0] == '+') {
+      forceIsTransportable = true;
+      rtpEncodingName = rtpEncodingName.Mid(1);
+    }
     return;
+  }
 
   PWaitAndSignal mutex(GetMediaFormatsListMutex());
   OpalMediaFormatList & registeredFormats = GetMediaFormatsList();
@@ -968,6 +993,27 @@ bool OpalMediaFormatInternal::ToCustomisedOptions()
 }
 
 
+bool OpalMediaFormatInternal::IsValid() const
+{
+  return rtpPayloadType < RTP_DataFrame::IllegalPayloadType && !formatName.IsEmpty();
+}
+
+
+bool OpalMediaFormatInternal::IsTransportable() const
+{
+  if (forceIsTransportable)
+    return true;
+
+  if (rtpPayloadType >= RTP_DataFrame::MaxPayloadType)
+    return false;
+
+  if (rtpPayloadType < RTP_DataFrame::LastKnownPayloadType)
+    return true;
+
+  return !rtpEncodingName.IsEmpty();
+}
+
+
 PStringToString OpalMediaFormatInternal::GetOptions() const
 {
   PWaitAndSignal m1(media_format_mutex);
@@ -1193,7 +1239,7 @@ bool OpalMediaFormatInternal::IsValidForProtocol(const PString & protocol) const
 
   // the protocol is only valid for SIP if the RTP name is not NULL
   if (protocol *= "sip")
-    return rtpEncodingName != NULL;
+    return (rtpEncodingName != NULL) || (forceIsTransportable);
 
   return true;
 }
@@ -1375,7 +1421,7 @@ const PString & OpalVideoFormat::TxKeyFramePeriodOption()         { static PStri
 const PString & OpalVideoFormat::RateControlEnableOption()        { static PString s = "Rate Control Enable";                        return s; }
 const PString & OpalVideoFormat::RateControlWindowSizeOption()    { static PString s = "Rate Control Window Size";                   return s; }
 const PString & OpalVideoFormat::RateControlMaxFramesSkipOption() { static PString s = "Rate Control Max Frames Skip";               return s; }
-
+const PString & OpalVideoFormat::RateControllerBitRateScalerOption() { static PString s = "Rate Controller Bit Rate Scaler";            return s; }
 
 OpalVideoFormat::OpalVideoFormat(const char * fullName,
                                  RTP_DataFrame::PayloadTypes rtpPayloadType,
@@ -1423,9 +1469,11 @@ OpalVideoFormatInternal::OpalVideoFormatInternal(const char * fullName,
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::MaxRxFrameWidthOption(),          false, OpalMediaOption::MinMerge,    maxFrameWidth,               16,  32767));
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::MaxRxFrameHeightOption(),         false, OpalMediaOption::MinMerge,    maxFrameHeight,              16,  32767));
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::TargetBitRateOption(),            false, OpalMediaOption::AlwaysMerge, maxBitRate,                  1000      ));
+  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::TxKeyFramePeriodOption(),         false, OpalMediaOption::AlwaysMerge, 125,                         0,   1000));
   AddOption(new OpalMediaOptionBoolean (OpalVideoFormat::RateControlEnableOption(),        false, OpalMediaOption::NoMerge,     false                                  ));
-  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControlWindowSizeOption(),    false, OpalMediaOption::NoMerge,     500,                         100, 10000));
-  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControlMaxFramesSkipOption(), false, OpalMediaOption::NoMerge,     1,                           1,   10   ));
+  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControlWindowSizeOption(),    false, OpalMediaOption::NoMerge,     5000,                        100,100000));
+  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControlMaxFramesSkipOption(), false, OpalMediaOption::NoMerge,     5,                           1,   10   ));
+  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControllerBitRateScalerOption(), false, OpalMediaOption::NoMerge,  100,                         50, 200));
 								
   // For video the max bit rate and frame rate is adjustable by user
   FindOption(OpalVideoFormat::MaxBitRateOption())->SetReadOnly(false);
@@ -1504,12 +1552,14 @@ OpalMediaFormatList & OpalMediaFormatList::operator-=(const OpalMediaFormatList
 }
 
 
-void OpalMediaFormatList::Remove(const PStringArray & mask)
+void OpalMediaFormatList::Remove(const PStringArray & maskList)
 {
+  PTRACE(4,"MediaFormat\tRemoving codecs " << setfill(',') << maskList);
+
   PINDEX i;
-  for (i = 0; i < mask.GetSize(); i++) {
+  for (i = 0; i < maskList.GetSize(); i++) {
     OpalMediaFormatList::const_iterator fmt;
-    while ((fmt = FindFormat(mask[i])) != end())
+    while ((fmt = FindFormat(maskList[i])) != end())
       erase(fmt);
   }
 }
@@ -1583,13 +1633,33 @@ static bool WildcardMatch(const PCaselessString & str, const PStringArray & wild
 
 OpalMediaFormatList::const_iterator OpalMediaFormatList::FindFormat(const PString & search, const_iterator iter) const
 {
-  PStringArray wildcards = search.Tokenise('*', true);
+  if (search.IsEmpty())
+    return end();
+
   if (iter == const_iterator())
     iter = begin();
-  while (iter != end()) {
-    if (WildcardMatch(iter->m_info->formatName, wildcards))
-      return iter;
-    ++iter;
+
+  bool negative = search[0] == '!';
+
+  PString adjustedSearch = search.Mid(negative ? 1 : 0);
+  if (adjustedSearch.IsEmpty())
+    return end();
+
+  if (adjustedSearch[0] == '@') {
+    OpalMediaType searchType = adjustedSearch.Mid(1);
+    while (iter != end()) {
+      if ((iter->GetMediaType() == searchType) != negative)
+        return iter;
+      ++iter;
+    }
+  }
+  else {
+    PStringArray wildcards = adjustedSearch.Tokenise('*', true);
+    while (iter != end()) {
+      if (WildcardMatch(iter->m_info->formatName, wildcards) != negative)
+        return iter;
+      ++iter;
+    }
   }
 
   return end();
@@ -1616,5 +1686,16 @@ void OpalMediaFormatList::Reorder(const PStringArray & order)
   AllowDeleteObjects();
 }
 
+bool OpalMediaFormatList::HasType(const OpalMediaType & type, bool mustBeTransportable) const
+{
+  OpalMediaFormatList::const_iterator format;
+  for (format = begin(); format != end(); ++format) {
+    if (format->GetMediaType() == type && (mustBeTransportable || format->IsTransportable()))
+      return true;
+  }
+
+  return false;
+}
+
 
 // End of File ///////////////////////////////////////////////////////////////
