@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21726 $
+ * $Revision: 22037 $
  * $Author: rjongbloed $
- * $Date: 2008-12-09 06:03:55 +0000 (Tue, 09 Dec 2008) $
+ * $Date: 2009-02-11 07:13:45 +0000 (Wed, 11 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -77,6 +77,7 @@ SIPHandler::SIPHandler(SIPEndPoint & ep,
                        const PTimeInterval & retryMax)
   : endpoint(ep)
   , m_transport(NULL)
+  , callID(SIPTransaction::GenerateCallID())
   , expire(expireTime > 0 ? expireTime : endpoint.GetNotifierTimeToLive().GetSeconds())
   , originalExpire(expire)
   , offlineExpire(offlineExpireTime)
@@ -125,8 +126,6 @@ SIPHandler::SIPHandler(SIPEndPoint & ep,
   authenticationAttempts = 0;
   authentication = NULL;
 
-  SIPTransaction::GenerateCallID(callID);
-
   expireTimer.SetNotifier(PCREATE_NOTIFIER(OnExpireTimeout));
 }
 
@@ -246,7 +245,6 @@ bool SIPHandler::WriteSIPHandler(OpalTransport & transport)
   }
 
   PTRACE(2, "SIP\tDid not start transaction on " << transport);
-  OnFailed(SIP_PDU::Local_TransportError);
   return false;
 }
 
@@ -288,6 +286,7 @@ PBoolean SIPHandler::SendRequest(SIPHandler::State s)
       if (WriteSIPHandler(*m_transport))
         return true;
     }
+    OnFailed(SIP_PDU::Local_TransportError);
     retryLater = true;
   }
 
@@ -394,7 +393,10 @@ void SIPHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response)
     case Subscribing :
     case Refreshing :
     case Restoring :
-      SetState(Subscribed);
+      if (expire == 0)
+        SetState(Unsubscribed);
+      else
+        SetState(Subscribed);
       break;
 
     default :
@@ -501,7 +503,9 @@ SIPRegisterHandler::SIPRegisterHandler(SIPEndPoint & endpoint, const SIPRegister
   , m_sequenceNumber(0)
 {
   // Put adjusted values back
-  m_parameters.m_addressOfRecord = GetAddressOfRecord().AsQuotedString()+";tag="+::OpalGloballyUniqueID().AsString();
+  SIPURL aor = GetAddressOfRecord();
+  aor.SetTag();
+  m_parameters.m_addressOfRecord = aor.AsQuotedString();
   m_parameters.m_registrarAddress = m_remoteAddress.AsQuotedString();
   m_parameters.m_expire = expire;
 
@@ -661,6 +665,7 @@ SIPSubscribeHandler::SIPSubscribeHandler(SIPEndPoint & endpoint, const SIPSubscr
                params.m_restoreTime, params.m_minRetryTime, params.m_maxRetryTime)
   , m_parameters(params)
   , m_unconfirmed(true)
+  , m_packageHandler(SIPEventPackageFactory::CreateInstance(params.m_eventPackage))
 {
   // Put possibly adjusted value back
   m_parameters.m_addressOfRecord = GetAddressOfRecord().AsString();
@@ -680,6 +685,7 @@ SIPSubscribeHandler::SIPSubscribeHandler(SIPEndPoint & endpoint, const SIPSubscr
 SIPSubscribeHandler::~SIPSubscribeHandler()
 {
   PTRACE(4, "SIP\tDeleting SIPSubscribeHandler " << GetAddressOfRecord());
+  delete m_packageHandler;
 }
 
 
@@ -765,14 +771,14 @@ void SIPSubscribeHandler::UpdateParameters(const SIPSubscribe::Params & params)
 
 void SIPSubscribeHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response)
 {
-  SIPHandler::OnReceivedOK(transaction, response);
-
   /* An "expire" parameter in the Contact header has no semantics
    * for SUBSCRIBE. RFC3265, 3.1.1.
    * An answer can only shorten the expires time.
    */
   SetExpire(response.GetMIME().GetExpires(originalExpire));
 
+  SIPHandler::OnReceivedOK(transaction, response);
+
   m_dialog.Update(response);
 
   response.GetMIME().GetProductInfo(m_productInfo);
@@ -818,15 +824,12 @@ PBoolean SIPSubscribeHandler::OnReceivedNOTIFY(SIP_PDU & request)
       SetExpire(expire.AsUnsigned());
   }
 
-  SIPEventPackage eventPackage = request.GetMIME().GetEvent();
-  SIPEventPackageHandler * packageHandler = SIPEventPackageFactory::CreateInstance(eventPackage);
-  if (packageHandler == NULL)
+  if (m_packageHandler == NULL)
     request.SendResponse(*m_transport, SIP_PDU::Failure_BadEvent, &endpoint);
-  else if (packageHandler->OnReceivedNOTIFY(*this, request))
+  else if (m_packageHandler->OnReceivedNOTIFY(*this, request))
     request.SendResponse(*m_transport, SIP_PDU::Successful_OK, &endpoint);
   else
     request.SendResponse(*m_transport, SIP_PDU::Failure_BadRequest, &endpoint);
-  delete packageHandler;
   return true;
 }
 
@@ -992,6 +995,12 @@ static void ParseParticipant(PXMLElement * participantElement, SIPDialogNotifica
 
 class SIPDialogEventPackageHandler : public SIPEventPackageHandler
 {
+public:
+  SIPDialogEventPackageHandler()
+    : m_dialogNotifyVersion(1)
+  {
+  }
+
   virtual PString GetContentType() const
   {
     return "application/dialog-info+xml";
@@ -1012,9 +1021,12 @@ class SIPDialogEventPackageHandler : public SIPEventPackageHandler
       return false;
 
     SIPDialogNotification info(rootElement->GetAttribute("entity"));
+    if (info.m_entity.IsEmpty())
+      return false;
 
-    PXMLElement * dialogElement = rootElement->GetElement("dialog");
-    if (dialogElement != NULL && !info.m_entity.IsEmpty()) {
+    PINDEX index = 0;
+    PXMLElement * dialogElement;
+    while ((dialogElement = rootElement->GetElement("dialog", index)) != NULL) {
       info.m_callId = dialogElement->GetAttribute("call-id");
       info.m_local.m_dialogTag = dialogElement->GetAttribute("local-tag");
       info.m_remote.m_dialogTag = dialogElement->GetAttribute("remote-tag");
@@ -1040,11 +1052,47 @@ class SIPDialogEventPackageHandler : public SIPEventPackageHandler
 
       ParseParticipant(dialogElement->GetElement("local"), info.m_local);
       ParseParticipant(dialogElement->GetElement("remote"), info.m_remote);
+      handler.GetEndPoint().OnDialogInfoReceived(info);
+      index++;
     }
 
-    handler.GetEndPoint().OnDialogInfoReceived(info);
+    if (index == 0)
+      handler.GetEndPoint().OnDialogInfoReceived(info);
     return true;
   }
+
+  virtual PString OnSendNOTIFY(SIPHandler & handler, const PObject * data)
+  {
+    PStringStream body;
+    body << "<?xml version=\"1.0\"?>\r\n"
+            "<dialog-info xmlns=\"urn:ietf:params:xml:ns:dialog-info\" version=\""
+         << m_dialogNotifyVersion++ << "\" state=\"partial\" entity=\""
+         << handler.GetAddressOfRecord() << "\">\r\n";
+
+    map<PString, SIPDialogNotification>::iterator iter;
+
+    const SIPDialogNotification * info = dynamic_cast<const SIPDialogNotification *>(data);
+    if (info != NULL) {
+      if (info->m_state != SIPDialogNotification::Terminated)
+        m_activeDialogs[info->m_callId] = *info;
+      else {
+        iter = m_activeDialogs.find(info->m_callId);
+        if (iter != m_activeDialogs.end())
+          m_activeDialogs.erase(iter);
+
+        body << *info;
+      }
+    }
+
+    for (iter = m_activeDialogs.begin(); iter != m_activeDialogs.end(); ++iter)
+      body << iter->second;
+
+    body << "</dialog-info>\r\n";
+    return body;
+  }
+
+  unsigned m_dialogNotifyVersion;
+  map<PString, SIPDialogNotification> m_activeDialogs;
 };
 
 static SIPEventPackageFactory::Worker<SIPDialogEventPackageHandler> dialogEventPackageHandler(SIPSubscribe::Dialog);
@@ -1052,6 +1100,16 @@ static SIPEventPackageFactory::Worker<SIPDialogEventPackageHandler> dialogEventP
 #endif // P_EXPAT
 
 
+SIPDialogNotification::SIPDialogNotification(const PString & entity)
+  : m_entity(entity)
+  , m_initiator(false)
+  , m_state(Terminated)
+  , m_eventType(NoEvent)
+  , m_eventCode(0)
+{
+}
+
+
 PString SIPDialogNotification::GetStateName(States state)
 {
   static const char * const Names[] = {
@@ -1086,6 +1144,69 @@ PString SIPDialogNotification::GetEventName(Events state)
 }
 
 
+static void OutputParticipant(ostream & strm, const char * name, const SIPDialogNotification::Participant & participant)
+{
+  if (participant.m_URI.IsEmpty())
+    return;
+
+  strm << "    <" << name << ">\r\n";
+
+  if (!participant.m_identity.IsEmpty()) {
+    strm << "      <identity";
+    if (!participant.m_display.IsEmpty())
+      strm << " display=\"" << participant.m_display << '"';
+    strm << '>' << participant.m_identity << "</identity>\r\n";
+  }
+
+  strm << "      <target uri=\"" << participant.m_URI << "\">\r\n";
+
+  if (participant.m_appearance >= 0)
+    strm << "        <param pname=\"appearance\" pval=\"" << participant.m_appearance << "\"/>\r\n"
+            "        <param pname=\"x-line-id\" pval=\"" << participant.m_appearance << "\"/>\r\n";
+
+  if (participant.m_byeless)
+    strm << "        <param pname=\"sip.byeless\" pval=\"true\"/>\r\n";
+
+  if (participant.m_rendering >= 0)
+    strm << "        <param pname=\"sip.rendering\" pval=\"" << (participant.m_rendering > 0 ? "yes" : "no") << "\"/>\r\n";
+
+  strm << "      </target>\r\n"
+       << "    </" << name << ">\r\n";
+}
+
+
+void SIPDialogNotification::PrintOn(ostream & strm) const
+{
+  if (m_dialogId.IsEmpty())
+    return;
+
+  // Start dialog XML tag
+  strm << "  <dialog id=\"" << m_dialogId << '"';
+  if (!m_callId)
+    strm << " call-id=\"" << m_callId << '"';
+  if (!m_local.m_dialogTag)
+    strm << " local-tag=\"" << m_local.m_dialogTag << '"';
+  if (!m_remote.m_dialogTag)
+    strm << " remote-tag=\"" << m_remote.m_dialogTag << '"';
+  strm << " direction=\"" << (m_initiator ? "initiator" : "receiver") << "\">\r\n";
+
+  // State XML tag & value
+  strm << "    <state";
+  if (m_eventType > SIPDialogNotification::NoEvent) {
+    strm << " event=\"" << GetEventName() << '"';
+    if (m_eventCode > 0)
+      strm << " code=\"" << m_eventCode << '"';
+  }
+  strm << '>' << GetStateName() << "</state>\r\n";
+
+  // Participant XML tags (local/remopte)
+  OutputParticipant(strm, "local", m_local);
+  OutputParticipant(strm, "remote", m_remote);
+
+  // Close out dialog tag
+  strm << "  </dialog>\r\n";
+}
+
 /////////////////////////////////////////////////////////////////////////
 
 SIPNotifyHandler::SIPNotifyHandler(SIPEndPoint & endpoint,
@@ -1096,11 +1217,18 @@ SIPNotifyHandler::SIPNotifyHandler(SIPEndPoint & endpoint,
   , m_eventPackage(eventPackage)
   , m_dialog(dialog)
   , m_reason(Deactivated)
+  , m_packageHandler(SIPEventPackageFactory::CreateInstance(eventPackage))
 {
   callID = m_dialog.GetCallID();
 }
 
 
+SIPNotifyHandler::~SIPNotifyHandler()
+{
+  delete m_packageHandler;
+}
+
+
 SIPTransaction * SIPNotifyHandler::CreateTransaction(OpalTransport & trans)
 {
   PString state;
@@ -1128,10 +1256,27 @@ PBoolean SIPNotifyHandler::SendRequest(SIPHandler::State state)
   // If times out, i.e. Refreshing, then this is actually a time out unsubscribe.
   if (state == Refreshing)
     m_reason = Timeout;
+
   return SIPHandler::SendRequest(state == Refreshing ? Unsubscribing : state);
 }
 
 
+bool SIPNotifyHandler::SendNotify(const PObject * body)
+{
+  if (m_packageHandler != NULL)
+    SetBody(m_packageHandler->OnSendNOTIFY(*this, body));
+  else if (body == NULL)
+    SetBody(PString::Empty());
+  else {
+    PStringStream str;
+    str << body;
+    SetBody(str);
+  }
+
+  return SendRequest(Subscribing);
+}
+
+
 /////////////////////////////////////////////////////////////////////////
 
 SIPPublishHandler::SIPPublishHandler(SIPEndPoint & endpoint,
