@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 22029 $
+ * $Author: csoutheren $
+ * $Date: 2009-02-10 05:25:00 +0000 (Tue, 10 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -59,23 +59,23 @@
 //  the following functions bind the media type factory to the SDP format types
 //
 
-PString OpalMediaType::GetSDPFromFromMediaType(const OpalMediaType & type)
-{
-  OpalMediaTypeDefinition * defn = type.GetDefinition();
-  if (defn == NULL)
-    return PString();
-  return defn->GetSDPType();
-}
-
-OpalMediaType OpalMediaType::GetMediaTypeFromSDP(const std::string & sdp)
+OpalMediaType OpalMediaType::GetMediaTypeFromSDP(const std::string & sdp, const std::string & transport)
 {
   OpalMediaTypeFactory::KeyList_T mediaTypes = OpalMediaTypeFactory::GetKeyList();
   OpalMediaTypeFactory::KeyList_T::iterator r;
+
   for (r = mediaTypes.begin(); r != mediaTypes.end(); ++r) {
     if (OpalMediaType::GetDefinition(*r)->GetSDPType() == sdp)
       return OpalMediaType(*r);
   }
 
+  std::string s = sdp +"|" + transport;
+
+  for (r = mediaTypes.begin(); r != mediaTypes.end(); ++r) {
+    if (OpalMediaType::GetDefinition(*r)->GetSDPType() == s)
+      return OpalMediaType(*r);
+  }
+
   return OpalMediaType();
 }
 
@@ -143,7 +143,7 @@ static PString GetConnectAddressString(const OpalTransportAddress & address)
 
   PIPSocket::Address ip;
   if (!address.IsEmpty() && address.GetIpAddress(ip) && ip.IsValid())
-    str << "IN IP" << ip.GetVersion() << ' ' << ip;
+    str << "IN IP" << ip.GetVersion() << ' ' << ip.AsString(PTrue);
   else
     str << "IN IP4 0.0.0.0";
 
@@ -153,38 +153,22 @@ static PString GetConnectAddressString(const OpalTransportAddress & address)
 
 /////////////////////////////////////////////////////////
 
-SDPMediaFormat::SDPMediaFormat(RTP_DataFrame::PayloadTypes pt, const char * _name)
-  : payloadType(pt)
+SDPMediaFormat::SDPMediaFormat(SDPMediaDescription & parent, RTP_DataFrame::PayloadTypes pt, const char * _name)
+  : m_parent(parent) 
+  , payloadType(pt)
   , clockRate(0)
   , encodingName(_name)
-  , nteSet(PTrue)
-  , nseSet(PTrue)
 {
-  if (encodingName == OpalRFC2833.GetEncodingName())
-    AddNTEString("0-15,32-49");
-#if OPAL_T38_CAPABILITY
-  else if (encodingName == OpalCiscoNSE.GetEncodingName())
-    AddNSEString("192,193");
-#endif
 }
 
 
-SDPMediaFormat::SDPMediaFormat(const OpalMediaFormat & fmt,
-                               RTP_DataFrame::PayloadTypes pt,
-                               const char * nxeString)
-  : mediaFormat(fmt)
-  , payloadType(pt)
+SDPMediaFormat::SDPMediaFormat(SDPMediaDescription & parent, const OpalMediaFormat & fmt)
+  : m_parent(parent) 
+  , mediaFormat(fmt)
+  , payloadType(fmt.GetPayloadType())
   , clockRate(fmt.GetClockRate())
   , encodingName(fmt.GetEncodingName())
-  , nteSet(PTrue)
-  , nseSet(PTrue)
 {
-  if (nxeString != NULL) {
-    if (encodingName *= "nse")
-      AddNSEString(nxeString);
-    else
-      AddNTEString(nxeString);
-  }
   if (fmt.GetMediaType() == OpalMediaType::Audio()) 
     parameters = PString(PString::Unsigned, fmt.GetOptionInteger(OpalAudioFormat::ChannelsOption()));
 }
@@ -194,20 +178,6 @@ void SDPMediaFormat::SetFMTP(const PString & str)
   if (str.IsEmpty())
     return;
 
-  if (encodingName == OpalRFC2833.GetEncodingName()) {
-    nteSet.RemoveAll();
-    AddNTEString(str);
-    return;
-  }
-
-#if OPAL_T38_CAPABILITY
-  else if (encodingName == OpalCiscoNSE.GetEncodingName()) {
-    nseSet.RemoveAll();
-    AddNSEString(str);
-    return;
-  }
-#endif
-
   fmtp = str;
   if (GetMediaFormat().IsEmpty()) // Use GetMediaFormat() to force creation of member
     return;
@@ -282,14 +252,6 @@ void SDPMediaFormat::SetFMTP(const PString & str)
 
 PString SDPMediaFormat::GetFMTP() const
 {
-  if (encodingName == OpalRFC2833.GetEncodingName())
-    return GetNTEString();
-
-#if OPAL_T38_CAPABILITY
-  if (encodingName == OpalCiscoNSE.GetEncodingName())
-    return GetNSEString();
-#endif
-
   if (GetMediaFormat().IsEmpty()) // Use GetMediaFormat() to force creation of member
     return fmtp;
 
@@ -314,90 +276,9 @@ PString SDPMediaFormat::GetFMTP() const
 }
 
 
-PString SDPMediaFormat::GetNTEString() const
-{
-  return GetNXEString(nteSet);
-}
-
-void SDPMediaFormat::AddNTEString(const PString & str)
-{
-  AddNXEString(nteSet, str);
-}
-
-void SDPMediaFormat::AddNTEToken(const PString & ostr)
-{
-  AddNXEToken(nteSet, ostr);
-}
-
-PString SDPMediaFormat::GetNSEString() const
-{
-  return GetNXEString(nseSet);
-}
-
-void SDPMediaFormat::AddNSEString(const PString & str)
-{
-  AddNXEString(nseSet, str);
-}
-
-void SDPMediaFormat::AddNSEToken(const PString & ostr)
-{
-  AddNXEToken(nseSet, ostr);
-}
-
-PString SDPMediaFormat::GetNXEString(POrdinalSet & nxeSet) const
-{
-  PString str;
-  PINDEX i = 0;
-  while (i < 255) {
-    if (!nxeSet.Contains(POrdinalKey(i)))
-      i++;
-    else {
-      PINDEX start = i++;
-      while (nxeSet.Contains(POrdinalKey(i)))
-        i++;
-      if (!str.IsEmpty())
-        str += ",";
-      str += PString(PString::Unsigned, start);
-      if (i > start+1)
-        str += PString('-') + PString(PString::Unsigned, i-1);
-    }
-  }
-
-  return str;
-}
-
-
-void SDPMediaFormat::AddNXEString(POrdinalSet & nxeSet, const PString & str)
-{
-  PStringArray tokens = str.Tokenise(",", PFalse);
-  PINDEX i;
-  for (i = 0; i < tokens.GetSize(); i++)
-    AddNXEToken(nxeSet, tokens[i]);
-}
-
-
-void SDPMediaFormat::AddNXEToken(POrdinalSet & nxeSet, const PString & ostr)
-{
-  PString str = ostr.Trim();
-  if (str[0] == ',')
-    str = str.Mid(1);
-  if (str.Right(1) == ",")
-    str = str.Left(str.GetLength()-1);
-  PINDEX pos = str.Find('-');
-  if (pos == P_MAX_INDEX)
-    nxeSet.Include(new POrdinalKey(str.AsInteger()));
-  else {
-    PINDEX from = str.Left(pos).AsInteger();
-    PINDEX to   = str.Mid(pos+1).AsInteger();
-    while (from <= to)
-      nxeSet.Include(new POrdinalKey(from++));
-  }
-}
-
-
 void SDPMediaFormat::PrintOn(ostream & strm) const
 {
-  PAssert(!encodingName.IsEmpty(), "SDPAudioMediaFormat encoding name is empty");
+  PAssert(!encodingName.IsEmpty(), "SDPMediaFormat encoding name is empty");
   mediaFormat.ToCustomisedOptions();
 
   PINDEX i;
@@ -427,11 +308,19 @@ void SDPMediaFormat::PrintOn(ostream & strm) const
   }
 }
 
+void SDPMediaFormat::UpdateMediaFormat() const
+{
+  mediaFormat = OpalMediaFormat(payloadType, clockRate, encodingName, "sip");
+  if (mediaFormat.IsEmpty())
+    mediaFormat = OpalMediaFormat(encodingName);
+}
 
 const OpalMediaFormat & SDPMediaFormat::GetMediaFormat() const
 {
   if (mediaFormat.IsEmpty()) {
-    mediaFormat = OpalMediaFormat(payloadType, clockRate, encodingName, "sip");
+
+    UpdateMediaFormat();
+
     PTRACE_IF(2, mediaFormat.IsEmpty(), "SDP\tCould not find media format for \""
               << encodingName << "\", pt=" << payloadType << ", clock=" << clockRate);
 
@@ -449,6 +338,9 @@ const OpalMediaFormat & SDPMediaFormat::GetMediaFormat() const
       if (!option.GetFMTPName().IsEmpty() && !option.GetFMTPDefault().IsEmpty())
         option.FromString(option.GetFMTPDefault());
     }
+
+    for (SDPBandwidth::const_iterator r = m_parent.GetBandwidth().begin(); r != m_parent.GetBandwidth().end(); ++r) 
+      mediaFormat.AddOption(new OpalMediaOptionString("Bandwidth-" & r->first, false, r->second), PTrue);
   }
 
   return mediaFormat;
@@ -467,7 +359,7 @@ bool SDPMediaFormat::ToNormalisedOptions()
   if (mediaFormat.ToNormalisedOptions())
     return true;
 
-  PTRACE(2, "SDP\tCould not normalise format \"" << GetEncodingName() << "\", pt=" << GetPayloadType() << ", removing.");
+  PTRACE(2, "SDP\tCould not normalise format \"" << encodingName << "\", pt=" << payloadType << ", removing.");
   return false;
 }
 
@@ -484,18 +376,6 @@ void SDPMediaFormat::SetPacketTime(const PString & optionName, unsigned ptime)
 }
 
 
-PBoolean SDPMediaDescription::SetTransportAddress(const OpalTransportAddress &t)
-{
-  PIPSocket::Address ip;
-  WORD port = 0;
-  if (transportAddress.GetIpAndPort(ip, port)) {
-    transportAddress = OpalTransportAddress(t, port);
-    return PTrue;
-  }
-  return PFalse;
-}
-
-
 //////////////////////////////////////////////////////////////////////////////
 
 unsigned & SDPBandwidth::operator[](const PString & type)
@@ -542,11 +422,26 @@ SDPMediaDescription::SDPMediaDescription(const OpalTransportAddress & address)
   portCount = 0;
 }
 
+PBoolean SDPMediaDescription::SetTransportAddress(const OpalTransportAddress &t)
+{
+  PIPSocket::Address ip;
+  WORD port = 0;
+  if (transportAddress.GetIpAndPort(ip, port)) {
+    transportAddress = OpalTransportAddress(t, port);
+    return PTrue;
+  }
+  return PFalse;
+}
 
 bool SDPMediaDescription::Decode(const PStringArray & tokens)
 {
+  if (tokens.GetSize() < 3) {
+    PTRACE(1, "SDP\tUnknown SDP media type " << tokens[0]);
+    return false;
+  }
+
   // parse the media type
-  mediaType = OpalMediaType::GetMediaTypeFromSDP(tokens[0]);
+  mediaType = OpalMediaType::GetMediaTypeFromSDP(tokens[0], tokens[2]);
   if (mediaType.empty()) {
     PTRACE(1, "SDP\tUnknown SDP media type " << tokens[0]);
     return false;
@@ -596,16 +491,22 @@ bool SDPMediaDescription::Decode(const PStringArray & tokens)
         transportAddress = OpalTransportAddress(ip, (WORD)port);
   }
 
+  CreateSDPMediaFormats(tokens);
+
+  return PTrue;
+}
+
+void SDPMediaDescription::CreateSDPMediaFormats(const PStringArray & tokens)
+{
   // create the format list
   for (PINDEX i = 3; i < tokens.GetSize(); i++) {
     SDPMediaFormat * fmt = CreateSDPMediaFormat(tokens[i]);
-    if (fmt == NULL) {
+    if (fmt != NULL) 
+      formats.Append(fmt);
+    else {
       PTRACE(2, "SDP\tCannot create SDP media format for port " << tokens[i]);
     }
-    formats.Append(fmt);
   }
-
-  return PTrue;
 }
 
 
@@ -691,7 +592,7 @@ void SDPMediaDescription::SetAttribute(const PString & attr, const PString & val
     return;
   }
 
-  // unknown attriutes
+  // unknown attributes
   PTRACE(2, "SDP\tUnknown media attribute " << attr);
   return;
 }
@@ -704,6 +605,7 @@ SDPMediaFormat * SDPMediaDescription::FindFormat(PString & params) const
   // extract the RTP payload type
   PINDEX pos = params.FindSpan("0123456789");
   if (pos == P_MAX_INDEX || isspace(params[pos])) {
+
     // find the format that matches the payload type
     RTP_DataFrame::PayloadTypes pt = (RTP_DataFrame::PayloadTypes)params.Left(pos).AsUnsigned();
     for (format = formats.begin(); format != formats.end(); ++format) {
@@ -714,9 +616,9 @@ SDPMediaFormat * SDPMediaDescription::FindFormat(PString & params) const
   else {
     // Check for it a format name
     pos = params.Find(' ');
-    PString fmtName = params.Left(pos);
+    PString encodingName = params.Left(pos);
     for (format = formats.begin(); format != formats.end(); ++format) {
-      if (format->GetEncodingName() == fmtName)
+      if (format->GetEncodingName() == encodingName)
         break;
     }
   }
@@ -884,7 +786,7 @@ void SDPMediaDescription::AddMediaFormat(const OpalMediaFormat & mediaFormat)
     }
   }
 
-  SDPMediaFormat * sdpFormat = new SDPMediaFormat(mediaFormat, payloadType);
+  SDPMediaFormat * sdpFormat = new SDPMediaFormat(*this, mediaFormat);
 
   ProcessMediaOptions(*sdpFormat, mediaFormat);
 
@@ -920,7 +822,7 @@ PCaselessString SDPRTPAVPMediaDescription::GetSDPTransportType() const
 
 SDPMediaFormat * SDPRTPAVPMediaDescription::CreateSDPMediaFormat(const PString & portString)
 {
-  return new SDPMediaFormat((RTP_DataFrame::PayloadTypes)portString.AsUnsigned());
+  return new SDPMediaFormat(*this, (RTP_DataFrame::PayloadTypes)portString.AsUnsigned());
 }
 
 
@@ -1078,7 +980,7 @@ PString SDPApplicationMediaDescription::GetSDPMediaType() const
 
 SDPMediaFormat * SDPApplicationMediaDescription::CreateSDPMediaFormat(const PString & portString)
 {
-  return new SDPMediaFormat(RTP_DataFrame::DynamicBase, portString);
+  return new SDPMediaFormat(*this, RTP_DataFrame::DynamicBase, portString);
 }
 
 PString SDPApplicationMediaDescription::GetSDPPortList() const
@@ -1284,6 +1186,8 @@ PBoolean SDPSessionDescription::Decode(const PString & str)
           case 'm' : // media name and transport address (mandatory)
             {
               if (currentMedia != NULL) {
+                PTRACE(3, "SDP\tParsed media session with " << currentMedia->GetSDPMediaFormats().GetSize()
+                                                            << " '" << currentMedia->GetSDPMediaType() << "' formats");
                 if (!currentMedia->PostDecode())
                   ok = false;
               }
@@ -1296,7 +1200,8 @@ PBoolean SDPSessionDescription::Decode(const PString & str)
               else
               {
                 // parse the media type
-                OpalMediaType mediaType = OpalMediaType::GetMediaTypeFromSDP(tokens[0]);
+                PString mt = tokens[0].ToLower();
+                OpalMediaType mediaType = OpalMediaType::GetMediaTypeFromSDP(tokens[0], tokens[2]);
                 if (mediaType.empty()) {
                   PTRACE(1, "SDP\tUnknown SDP media type " << tokens[0]);
                   currentMedia = NULL;
@@ -1315,8 +1220,6 @@ PBoolean SDPSessionDescription::Decode(const PString & str)
                     }
                     else if (currentMedia->Decode(tokens)) {
                       mediaDescriptions.Append(currentMedia);
-                      PTRACE(3, "SDP\tAdding media session with " << currentMedia->GetSDPMediaFormats().GetSize()
-                                                                  << ' ' << currentMedia->GetSDPMediaType() << " formats");
                     }
                     else {
                       delete currentMedia;
@@ -1336,6 +1239,8 @@ PBoolean SDPSessionDescription::Decode(const PString & str)
   }
 
   if (currentMedia != NULL) {
+    PTRACE(3, "SDP\tParsed media session with " << currentMedia->GetSDPMediaFormats().GetSize()
+                                                << " '" << currentMedia->GetSDPMediaType() << "' formats");
     if (!currentMedia->PostDecode())
       ok = false;
   }
