@@ -23,9 +23,9 @@
  *
  * Contributor(s): Vyacheslav Frolov.
  *
- * $Revision: 21865 $
+ * $Revision: 22005 $
  * $Author: rjongbloed $
- * $Date: 2008-12-23 05:18:47 +0000 (Tue, 23 Dec 2008) $
+ * $Date: 2009-02-06 02:52:21 +0000 (Fri, 06 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -505,7 +505,8 @@ PBoolean OpalFaxMediaStream::Close()
 
 PBoolean OpalFaxMediaStream::IsSynchronous() const
 {
-  return true;
+  // Make sure we don't get a jitter buffer by saying we are asynchronous when a sink stream
+  return IsSource();
 }
 
 PString OpalFaxMediaStream::GetSpanDSPCommandLine(OpalFaxCallInfo & info)
@@ -728,11 +729,15 @@ PBoolean OpalFaxEndPoint::MakeConnection(OpalCall & call,
     return false;
   }
 
+  OpalConnection::StringOptions localOptions;
   if (stringOptions == NULL)
-    stringOptions = new OpalConnection::StringOptions;
+    stringOptions = &localOptions;
+
   if ((*stringOptions)("stationid").IsEmpty())
     stringOptions->SetAt("stationid", stationId);
 
+  stringOptions->SetAt("Disable-Jitter", "1");
+
   return AddConnection(CreateConnection(call, userData, stringOptions, filename, receive,
                                         remoteParty.Left(prefixLength) *= m_t38Prefix));
 }
@@ -751,7 +756,7 @@ OpalFaxConnection::OpalFaxConnection(OpalCall        & call,
   , m_filename(filename)
   , m_receive(receive)
 {
-  PTRACE(3, "FAX\tCreated FAX connection with token '" << callToken << "'");
+  PTRACE(3, "FAX\tCreated FAX connection with token \"" << callToken << '"');
 
   m_faxStopped.SetNotifier(PCREATE_NOTIFIER(OnFaxStoppedTimeout));
 }
@@ -782,8 +787,6 @@ PBoolean OpalFaxConnection::SetUpConnection()
   if (ownerCall.GetConnection(0) == this) {
     SetPhase(SetUpPhase);
 
-    OnApplyStringOptions();
-
     if (!OnIncomingConnection(0, NULL)) {
       Release(EndedByCallerAbort);
       return false;
@@ -874,7 +877,7 @@ OpalT38Connection::OpalT38Connection(OpalCall        & call,
   , m_syncMode(Mode_Timeout)
   , m_faxMode(false)
 {
-  PTRACE(3, "FAX\tCreated T.38 connection with token '" << callToken << "'");
+  PTRACE(3, "FAX\tCreated T.38 connection");
 }
 
 
